const { prompt, Select } = require('enquirer');
const data = require('./../utils/data');
const { yellow: y } = require('chalk');
const { uuidGenerator } = require('./../utils/helpers');
const alert = require('cli-alerts');
const { teamMainMenu } = require('./team');
const Helper = require('../utils/helpers');
const event = require('../utils/event');
const Table = require('cli-table');
const { selectProjectForm, confirmToggle } = require('./common');

const projectMainMenu = async () => {
	return await new Promise(async (resolve, reject) => {
		const actionOptions = {
			addNewProject: '📦 Add New Project',
			viewProjects: '📋 View Projects',
			deleteProject: '❗️ Delete Project',
			projectTeamMembers: '👥 Project Team Members',
			back: 'Back'
		};
		const prompt = new Select({
			name: 'action',
			message: 'What you want to do? - Projects',
			choices: Object.values(actionOptions)
		});
		await prompt.run().then(async answer => {
			switch (answer) {
				case actionOptions.addNewProject:
					resolve(await addNewProject());
					break;
				case actionOptions.viewProjects:
					resolve(await viewProjects());
					break;
				case actionOptions.deleteProject:
					resolve(await deleteProject());
					break;
				case actionOptions.projectTeamMembers:
					resolve(await teamMainMenu());
					break;
				case actionOptions.back:
					resolve({ success: false, action: event.back });
					break;
			}
		});
	});
};

const addNewProject = async () => {
	return await new Promise(async resolve => {
		console.log(`${y('Add new project')}`);
		let responses = await prompt([
			{
				type: 'input',
				name: 'projectName',
				message: 'Project name',
				validate: Helper.getValidate()
			},
			{
				type: 'input',
				name: 'projectOwner',
				message: 'Project owner',
				validate: Helper.getValidate()
			},
			{
				type: 'input',
				name: 'projectOwnerEmail',
				message: 'Project owner `Teams` email',
				validate: Helper.getValidate()
			}
		]);
		let uuid = uuidGenerator();
		await data.mainStore.union(`projects`, {
			name: responses.projectName,
			owner: responses.projectOwner,
			uuid: uuid,
			members: [
				{
					name: responses.projectOwner,
					email: responses.projectOwnerEmail,
					username: Helper.extractUsername(
						responses.projectOwnerEmail
					),
					role: ['owner'],
					uuid: uuidGenerator()
				}
			]
		});
		resolve({
			success: true,
			action: event.addNewProject,
			name: responses.projectName
		});
	});
};

const viewProjects = async () => {
	return await new Promise(async resolve => {
		let projects = await data.getProjects();

		const table = new Table({
			head: ['Project Name', 'Members']
		});

		projects.forEach(project => {
			let projectMembers =
				project?.members?.map(
					member =>
						`${member.name} (${Helper.objToString(member.role)}),\n`
				) || [];

			table.push([project.name, Helper.objToString(projectMembers)]);
		});

		console.log(table.toString());
		let answers = await prompt([
			{
				type: 'toggle',
				name: 'sure',
				message: 'Back to Main menu?',
				initial: true,
				enabled: 'Yes',
				disabled: 'No'
			}
		]);
		if (answers.sure) {
			resolve({ success: false, action: event.back });
		}
	});
};

const deleteProject = async () => {
	return await new Promise(async (resolve, reject) => {
		let project = await data.getProjects();
		const selectProject = await selectProjectForm(project);

		if (selectProject.project.uuid === 'cancel') {
			resolve({ success: false, action: event.deleteProject });
			return;
		}

		let confirmAction = await confirmToggle();

		if (!confirmAction.sure) {
			resolve({ success: false, action: event.deleteProject });
			return;
		}
		project = project.filter(
			item => item.uuid !== selectProject.project.uuid
		);
		await data.mainStore.set('projects', project);
		resolve({ success: true, action: event.deleteProject });
	});
};

module.exports = {
	projectMainMenu,
	addNewProject,
	deleteProject
};
